<?php


class BackModuleEntityRepository
{
    private $requetor;
    private $bases;

    public function __construct(SiteSQL $requetor, array $bases = [])
    {
        $this->requetor = $requetor;
        $this->bases = $bases;
    }

    public function create($formvars = array(), $table = null, $langue = true)
    {
        if (empty($formvars['langue_id'])) {
            $formvars['langue_id'] = LANGUE;
        }

        $tables = (null === $table) ? $this->bases : (array) $table;


        foreach ($tables as $table) {
            $fields = [];
            $values = [];

            foreach ($this->getTableDescription($table) as $tab_champ) {
                if (isset($formvars[strtolower($tab_champ['Field'])])) {
                    $fields[] = $tab_champ['Field'];
                    $values[] = $this->getSqlFormatedValue($tab_champ['Type'], $formvars[strtolower($tab_champ['Field'])]);
                }
            }

            $this->requetor->query(
                sprintf(
                    'INSERT INTO %s (%s) VALUES (%s)',
                    $table,
                    implode(',', $fields),
                    implode(',', $values)
                )
            );

            $lastId = $this->requetor->db->insert_id;
        }

        $formvars['back_id'] = $lastId;

        if ($langue) {
            $this->updateLangue($formvars, $tables[0]);
        }

        return $lastId;
    }

    public function create_multi($formvars = array(), $table = null, $langue = true)
    {
        if (empty($formvars['langue_id'])) {
            $formvars['langue_id'] = LANGUE;
        }

        $tables = (null === $table) ? $this->bases : (array) $table;

        foreach ($tables as $table) {
            $fields = [];
            $values = [];

            foreach ($this->getTableDescription($table) as $tab_champ) {
                if ($tab_champ['Key'] === 'PRI') {
                    $key = $tab_champ['Field'];
                }

                if (isset($formvars[strtolower($tab_champ['Field'])])) {
                    $fields[] = $tab_champ['Field'];
                    $values[] = $this->getSqlFormatedValue($tab_champ['Type'], $formvars[strtolower($tab_champ['Field'])]);
                }
            }

            $this->requetor->query(
                sprintf(
                    'INSERT INTO %s (%s) VALUES (%s)',
                    $table,
                    implode(',', $fields),
                    implode(',', $values)
                )
            );

            $lastId = $this->requetor->db->insert_id;

            $formvars[$key] = $lastId;


            if (!isset($parentKey)) {
                $parentKey = $lastId;
            }
        }

        $formvars['back_id'] = $parentKey;

        if ($langue) {
            $this->updateLangue($formvars, $tables[0]);
        }

        return $parentKey;
    }

    public function update($formvars = array(), $table = null, $langue = true) : void
    {
        $tables = (null === $table) ? $this->bases : (array) $table;

        foreach ($tables as $table) {
            $sqlParts = [];
            $key = '';

            foreach ($this->getTableDescription($table) as $tab_champ) {
                if ($tab_champ['Key'] === 'PRI') {
                    $key = $tab_champ['Field'];
                }

                if (isset($formvars[strtolower($tab_champ['Field'])])) {
                    $sqlParts[] = $tab_champ['Field'] . ' = ' . $this->getSqlFormatedValue($tab_champ['Type'], $formvars[strtolower($tab_champ['Field'])]);
                }

            }

            $this->requetor->query(
                sprintf(
                    'UPDATE %s SET %s WHERE %s="%s"',
                    $table,
                    implode(',', $sqlParts),
                    $key,
                    $formvars['back_id']
                )
            );
        }

        if ($langue) {
            $this->updateLangue($formvars, $tables[0]);
        }
    }

    public function update_multi($formvars = array(), $table = null, $langue = true)
    {
        $tables = (null === $table) ? $this->bases : (array) $table;

        $parentKey = false;

        $i = 0;
        foreach ($tables as $table) {
            $sqlParts = [];

            foreach ($this->getTableDescription($table) as $tab_champ) {
                if ($tab_champ['Key'] === 'PRI') {
                    $key = $tab_champ['Field'];
                }

                if (isset($formvars[strtolower($tab_champ['Field'])])) {
                    $sqlParts[] = $tab_champ['Field'] . ' = ' . $this->getSqlFormatedValue($tab_champ['Type'], $formvars[strtolower($tab_champ['Field'])]);
                }

            }

            if (!$parentKey) {
                $parentKey = $key;
            }

            $query = sprintf(
                'UPDATE %s SET %s',
                $table,
                implode(',', $sqlParts)
            );


            if ($parentKey && strpos($table, '_nom') !== false ) {
                $query .= sprintf(' WHERE %s = "%s" AND langue_id = "%s"', $parentKey, $formvars['back_id'], LANGUE);
            } else {
                $query .= sprintf(' WHERE %s = "%s"', $key, $formvars['back_id']);
            }

            if ($i > 0) {
                $this->requetor->query($query);
            }
            $i++;
        }

        if ($langue) {
            $this->updateLangue($formvars, $tables[0]);
        }
    }

    public function updateLangue($formvars, $table)
    {
        $lang = [];

        foreach ($formvars as $k => $v) {
            if (strpos($k, '|') !== false) {
                [$langue_id, $champ] = explode('|', $k);
            } else {
                $langue_id = $k;
                $champ = '';
            }

            if (!empty($langue_id) && !empty($champ)) {
                $lang[$langue_id][$champ] = $v;
            }
        }

        if ($lang) {
            foreach ($lang as $k => $v) {
                $sql = sprintf(
                    'SELECT %s_nom_id as id FROM %s_nom WHERE langue_id = %s AND %s_id = %s',
                    $table,
                    $table,
                    $k,
                    $table,
                    $formvars['back_id']
                );

                $this->requetor->query($sql, SQL_INIT, SQL_ASSOC);
                $nbResults = $this->requetor->record['id'];

                if ($nbResults) {
                    $v['back_id'] = $nbResults;
                    $this->update($v, $table . '_nom', false);
                } else {
                    $v['langue_id'] = $k;
                    $v[$table . '_id'] = $formvars['back_id'];
                    $this->create($v,  $table . '_nom', false);
                }
            }
        }
    }

    public function delete($formvars = array())
    {
        return $this->requetor->query(sprintf('DELETE FROM %s WHERE %s_id = %s LIMIT 1', $this->bases[0], $this->bases[0], $formvars['back_id']));
    }

    private function getTableDescription($table)
    {
        $this->requetor->query("DESC $table", SQL_ALL, SQL_ASSOC);
        return $this->requetor->record;
    }

    private function getSqlFormatedValue($fieldType, $value) : string
    {
        if ($fieldType === 'date' && $value) {
            [$day, $month, $year] = explode('/', $value);
            $value = sprintf('%s-%s-%s', $year, $month, $day);
        }

        if ($fieldType === 'datetime' && $value) {
            [$date, $time] = explode(' ', $value);
            [$day, $month, $year] = explode('/', $date);
            $value = sprintf('%s-%s-%s %s', $year, $month, $day, $time);
        }

        return ($value == 'NULL') ? $value : '"' . $this->requetor->db->real_escape_string($value) .'"';
    }
}
